@@ -39,6 +39,7 @@ class CMouseAddTextBox;
 class CMouseAddDistance;
 class CMouseOverlay;
 class CMouseColorPicker;
+class CMouseSelWpt;
 class CWpt;
 class QLabel;
 class QSize;
@@ -51,6 +52,10 @@ class CPlot;
 class CTrack;
 
 #define PAINT_ROUNDED_RECT(p,r) p.drawRoundedRect(r,5,5)
+#define COMPASS_H 60
+#define COMPASS_W 30
+#define COMPASS_OFFSET_X 50
+#define COMPASS_OFFSET_Y 100
 
 
 inline  void USE_ANTI_ALIASING(QPainter& p, bool useAntiAliasing)
@@ -64,7 +69,7 @@ class CCanvas : public QWidget
     Q_OBJECT;
     public:
 
-        enum move_direction_e {eMoveLeft, eMoveRight, eMoveUp, eMoveDown, eMoveCenter};
+        enum move_direction_e {eMoveLeft, eMoveRight, eMoveUp, eMoveDown, eMoveLeftSmall, eMoveRightSmall, eMoveUpSmall, eMoveDownSmall, eMoveCenter};
 
         CCanvas(QWidget * parent);
         virtual ~CCanvas();
@@ -78,17 +83,14 @@ class CCanvas : public QWidget
             , eMouseEditWpt      ///< use mouse to select waypoints
             , eMouseMoveWpt      ///< use mouse to drag-n-drop waypoints
             , eMouseMoveRefPoint ///< use mouse to drag-n-drop reference points
-            //, eMouseSearchOC    ///< use mouse to define a search radius for open caching
             , eMouseCutTrack     ///< use mouse to cut a track into two pieces
             , eMouseSelTrack     ///< use mouse to select points of a track by a rectangle
-            //, eMouseEditRte     ///< use mouse to define a new route polyline
-            //, eMouseMoveRte     ///< use mouse to move route points
-            //, eMouseDelRte      ///< use mouse to delete route points
             , eMouseAddText      ///< use mouse to define a new text field on the map
             , eMouseAddTextBox   ///< use mouse to define a new text field with anchor on the map
             , eMouseAddDistance  ///< use mouse to define a new distance polygon
             , eMouseOverlay      ///< use mouse to change overlays
             , eMouseColorPicker  ///< use mouse to pick a color from map
+            , eMouseSelWpt       ///< use mouse to select waypoints in a radius
         };
 
         /// zoom in/out with a given point as static
@@ -129,6 +131,8 @@ class CCanvas : public QWidget
         static QBrush brushBackWhite;
         static QBrush brushBackYellow;
 
+        void setFadingMessage(const QString& msg);
+
         signals:
         void sigResize(const QSize& size);
 
@@ -137,6 +141,7 @@ class CCanvas : public QWidget
         void slotHighlightTrack(CTrack * track);
         void slotTrackChanged();
         void slotFocusTrackPoint(double dist);
+        void slotFadingMessage();
 
     protected:
         void paintEvent(QPaintEvent * e);
@@ -159,6 +164,7 @@ class CCanvas : public QWidget
         void drawRefPoints(QPainter& p);
         void drawScale(QPainter& p);
         void drawCompass(QPainter& p);
+        void drawFadingMessage(QPainter& p);
 
     private:
         friend class CStatusCanvas;
@@ -180,16 +186,20 @@ class CCanvas : public QWidget
         CMouseAddDistance * mouseAddDistance;
         CMouseOverlay * mouseOverlay;
         CMouseColorPicker * mouseColorPicker;
-        bool cursorFocus;
+        CMouseSelWpt * mouseSelWpt;
 
         /// current mouse mode
         mouse_mode_e mouseMode;
 
         /// current mouse position
         QPoint posMouse;
-
         QLabel * info;
-
         CPlot * profile;
+
+        QTimer * timerFadingMessage;
+        QString fadingMessage;
+
+        // work around possible Qt/OS X bug
+        bool contextMenuActive;
 };
 #endif                           //CCANVAS_H
