@@ -22,6 +22,7 @@
 #include "CTrackDB.h"
 #include "GeoMath.h"
 #include "IUnit.h"
+#include "CSettings.h"
 
 #include <QtGui>
 
@@ -38,7 +39,7 @@ CDlgTrackFilter::CDlgTrackFilter(CTrack &track, QWidget * parent)
 {
     setupUi(this);
 
-    QSettings cfg;
+    SETTINGS;
 
     checkReduceDataset->setChecked(false);
     checkModifyTimestamps->setChecked(false);
@@ -66,7 +67,7 @@ CDlgTrackFilter::CDlgTrackFilter(CTrack &track, QWidget * parent)
         tabTimestamp->setEnabled(false);
         radioTimedelta->setEnabled(false);
         spinTimedelta->setEnabled(false);
-        qDebug() << "Track has no timestamps that could be modified.";
+//        qDebug() << "Track has no timestamps that could be modified.";
     }
     else
     {
@@ -82,6 +83,9 @@ CDlgTrackFilter::CDlgTrackFilter(CTrack &track, QWidget * parent)
 
     spinTimedelta->setValue(cfg.value("trackfilter/timedelta",5).toInt());
 
+    quint32 cnt = track.getMedianFilterCount();
+    checkMedian->setText(tr("Smooth profile (Median filter, %1 tabs)").arg(5 + (cnt<<1)));
+
     // user-tunable elements on "Modify Timestamps" tab
     connect(buttonReset1stOfMonth, SIGNAL(clicked()), this, SLOT(slotReset1stOfMonth()));
     connect(buttonResetEpoch, SIGNAL(clicked()), this, SLOT(slotResetEpoch()));
@@ -123,7 +127,7 @@ void CDlgTrackFilter::accept()
         reduceDataset(&track);
     }
 
-    QSettings cfg;
+    SETTINGS;
     cfg.setValue("trackfilter/distance",spinDistance->value());
     cfg.setValue("trackfilter/azimuthdelta",spinAzimuthDelta->value());
     cfg.setValue("trackfilter/timedelta",spinTimedelta->value());
@@ -169,7 +173,7 @@ void CDlgTrackFilter::splitTrack(CTrack * trk)
     {
 
         progress.setValue(totalCnt++);
-        qApp->processEvents(QEventLoop::AllEvents, 100);
+        qApp->processEvents();
 
         *track1 << *trkpt;
         if(++trkptCnt >= chunk)
@@ -245,7 +249,7 @@ void CDlgTrackFilter::modifyTimestamp(CTrack * trk)
                 }
                 ++trkpt;
                 progress.setValue(i);
-                qApp->processEvents(QEventLoop::AllEvents, 100);
+                qApp->processEvents();
                 ++i;
                 if (progress.wasCanceled())
                 {
@@ -264,8 +268,6 @@ void CDlgTrackFilter::modifyTimestamp(CTrack * trk)
 
 void CDlgTrackFilter::reduceDataset(CTrack * trk)
 {
-
-
     if(checkReduceDataset->isChecked())
     {
         QList<CTrack::pt_t>& trkpts = trk->getTrackPoints();
@@ -300,7 +302,7 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
                 ++trkpt;
                 ++i;
                 progress.setValue(i);
-                qApp->processEvents(QEventLoop::AllEvents, 100);
+                qApp->processEvents();
                 if (progress.wasCanceled())
                     break;
             }
@@ -318,7 +320,7 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
             {
                 min_distance *= 0.3048f;
             }
-            XY p1, p2;
+            projXY p1, p2;
             p1.u = DEG_TO_RAD * trkpt->lon;
             p1.v = DEG_TO_RAD * trkpt->lat;
             ++trkpt;
@@ -329,6 +331,8 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
 
             double lastEle = trkpt->ele;
 
+            bool checkAzimuth = checkAzimuthDelta->isEnabled() && checkAzimuthDelta->isChecked();
+
             while(trkpt != trkpts.end())
             {
 
@@ -341,7 +345,7 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
 
                 double delta = distance(p1,p2,a1,a2);
 
-                if(checkAzimuthDelta->isEnabled() && checkAzimuthDelta->isChecked())
+                if(checkAzimuth)
                 {
                     if (abs(trkpt->azimuth) <= 180)
                     {
@@ -362,7 +366,7 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
 
                 double deltaEle = abs(lastEle - trkpt->ele);
 
-                if( delta < min_distance || (AzimuthDelta < minAzimuthDelta && checkAzimuthDelta->isEnabled() && checkAzimuthDelta->isChecked()))
+                if (delta < min_distance || (checkAzimuth && AzimuthDelta < minAzimuthDelta))
                 {
                     if(deltaEle < 3)
                     {
@@ -374,7 +378,7 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
                 {
                     p1 = p2;
                     progress.setValue(i);
-                    qApp->processEvents(QEventLoop::AllEvents, 100);
+                    qApp->processEvents();
                     if(AzimuthDelta >= minAzimuthDelta)
                     {
                         lastAzimuth = trkpt->azimuth;
@@ -393,33 +397,11 @@ void CDlgTrackFilter::reduceDataset(CTrack * trk)
             QApplication::restoreOverrideCursor();
         }
 
-        // can be done in parallel to othe reductions
+        // can be done in parallel to other reductions
         if(checkMedian->isChecked())
         {
             QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-            QList<float> window;
-            window << 0.0 << 0.0 << 0.0 << 0.0 << 0.0;
-
-
-            for(int i = 2; i < trkpts.size()-2; i++)
-            {
-                // apply median filter over all trackpoints
-                window[0] = trkpts[i - 2]._ele;
-                window[1] = trkpts[i - 1]._ele;
-                window[2] = trkpts[i]._ele;
-                window[3] = trkpts[i + 1]._ele;
-                window[4] = trkpts[i + 2]._ele;
-
-                qSort(window);
-                trkpts[i].ele = window[2];
-
-                progress.setValue(i);
-                qApp->processEvents(QEventLoop::AllEvents, 100);
-                if (progress.wasCanceled())
-                {
-                    break;
-                }
-            }
+            trk->medianFilter(progress);
             QApplication::restoreOverrideCursor();
         }
         progress.setValue(npts);
@@ -438,8 +420,8 @@ void CDlgTrackFilter::slotReset1stOfMonth()
     int offset = (day - 1) * 86400 + hour * 3600;
     QDateTime tn = t.addSecs(-offset);
 
-    qDebug() << "Resetting starttime:" << t.toString("yyyy-MM-dd'T'hh:mm:ss'Z'")
-        << "to:" << tn.toString("yyyy-MM-dd'T'hh:mm:ss'Z'");
+//    qDebug() << "Resetting starttime:" << t.toString("yyyy-MM-dd'T'hh:mm:ss'Z'")
+//        << "to:" << tn.toString("yyyy-MM-dd'T'hh:mm:ss'Z'");
 
     datetimeStartTime->setDateTime(tn);
 
@@ -451,8 +433,8 @@ void CDlgTrackFilter::slotResetEpoch()
 {
     QDateTime t = datetimeStartTime->dateTime();
 
-    qDebug() << "Resetting starttime:" << t.toString("yyyy-MM-dd'T'hh:mm:ss'Z'")
-        << "to epoch";
+//    qDebug() << "Resetting starttime:" << t.toString("yyyy-MM-dd'T'hh:mm:ss'Z'")
+//        << "to epoch";
 
     QDateTime tn;
     radioUTC->setChecked(true);
@@ -471,7 +453,7 @@ void CDlgTrackFilter::slotDateTimeChanged(const QDateTime &tn)
 {
     checkModifyTimestamps->setChecked(true);
 
-    qDebug() << "Resetting starttime to:" << tn.toString("yyyy-MM-dd'T'hh:mm:ss'Z'");
+//    qDebug() << "Resetting starttime to:" << tn.toString("yyyy-MM-dd'T'hh:mm:ss'Z'");
 }
 
 
