@@ -31,12 +31,13 @@
 #include "CRoute.h"
 #include "CMegaMenu.h"
 #include "CDlgConvertToTrack.h"
+#include "CResources.h"
 
 #include <QtGui>
 
 bool COverlayDistance::showBullets = true;
 
-bool operator==(const XY& p1, const XY& p2)
+static bool operator==(const projXY& p1, const projXY& p2)
 {
     return (p1.u == p2.u) && (p1.v == p2.v);
 }
@@ -47,6 +48,8 @@ COverlayDistance::COverlayDistance(const QString& name, const QString& comment,
 : IOverlay(parent, "Distance", ":/icons/iconDistance16x16.png")
 , points(pts)
 , thePoint(0)
+, thePointBefor(0)
+, thePointAfter(0)
 , distance(0)
 , speed(speed)
 , doSpecialCursor(false)
@@ -77,10 +80,11 @@ COverlayDistance::COverlayDistance(const QString& name, const QString& comment,
     if(pts.size() == 1)
     {
         points.append(points[0]);
-        thePoint    = &points[1];
-        doMove      = true;
-        addType     = eAtEnd;
-        doFuncWheel = false;
+        thePointBefor   = &points[0];
+        thePoint        = &points[1];
+        doMove          = true;
+        addType         = eAtEnd;
+        doFuncWheel     = false;
     }
 }
 
@@ -94,7 +98,7 @@ COverlayDistance::~COverlayDistance()
 void COverlayDistance::save(QDataStream& s)
 {
     s << name << comment << points.size();
-    XY pt;
+    projXY pt;
     foreach(pt, points)
     {
         s << pt.u << pt.v;
@@ -187,13 +191,15 @@ bool COverlayDistance::isCloseEnough(const QPoint& pt)
         return true;
     }
 
-    thePoint = 0;
+    thePoint        = 0;
+    thePointBefor   = 0;
+    thePointAfter   = 0;
 
     double ref  = doFuncWheel ? (35.0 * 35.0) : (8.0 * 8.0);
     double dist = ref;
     while(p != points.end())
     {
-        XY pt1 = *p;
+        projXY pt1 = *p;
         map.convertRad2Pt(pt1.u, pt1.v);
 
         double d = (pt.x() - pt1.u) * (pt.x() - pt1.u) + (pt.y() - pt1.v) * (pt.y() - pt1.v);
@@ -203,27 +209,32 @@ bool COverlayDistance::isCloseEnough(const QPoint& pt)
 
             if(p != points.begin())
             {
-                XY p1 = *p;
-                XY p2 = *(p - 1);
+                projXY p1 = *p;
+                projXY p2 = *(p - 1);
 
                 map.convertRad2M(p1.u, p1.v);
                 map.convertRad2M(p2.u, p2.v);
 
-                anglePrev = atan2((p2.v - p1.v) , (p2.u - p1.u)) * 180/PI;
+                anglePrev = atan2((p2.v - p1.v) , (p2.u - p1.u)) * 180/M_PI;
+
+                thePointBefor = &(*(p-1));
             }
             else
             {
                 anglePrev = 1000;
             }
+
             if((p + 1) != points.end())
             {
-                XY p1 = *p;
-                XY p2 = *(p + 1);
+                projXY p1 = *p;
+                projXY p2 = *(p + 1);
 
                 map.convertRad2M(p1.u, p1.v);
                 map.convertRad2M(p2.u, p2.v);
 
-                angleNext = atan2((p2.v - p1.v) , (p2.u - p1.u)) * 180/PI;
+                angleNext = atan2((p2.v - p1.v) , (p2.u - p1.u)) * 180/M_PI;
+
+                thePointAfter = &(*(p+1));
 
             }
             else
@@ -240,6 +251,51 @@ bool COverlayDistance::isCloseEnough(const QPoint& pt)
 }
 
 
+void COverlayDistance::keyPressEvent(QKeyEvent * e)
+{
+    if (e->key() == Qt::Key_Backspace)
+    {
+        if (!doFuncWheel && addType != eNone && thePoint && points.size() > 1)
+        {
+            int idx;
+            switch (addType)
+            {
+            case eBefore:
+                idx = points.indexOf(*thePoint);
+                if (idx < points.size())
+                {
+                    points.removeAt(idx+1);
+                    thePointAfter = (idx+1 < points.size()) ? &(points[idx+1]) : 0;
+                }
+                break;
+            case eAfter:
+                idx = points.indexOf(*thePoint);
+                if (idx > 0)
+                {
+                    points.removeAt(idx-1);
+                    thePointBefor = (idx > 1) ? &(points[idx-2]) : 0;
+                }
+                break;
+            case eAtEnd:
+                points.removeLast();
+                thePointBefor = (points.size() > 1) ? &(*(points.end() - 2)) : 0;
+                thePoint      = &points.last();
+                break;
+            default:
+                break;
+            }
+            calcDistance();
+            emit sigChanged();
+            QPoint pos = theMainWindow->getCanvas()->mapFromGlobal(QCursor::pos());
+            QMouseEvent * ev = new QMouseEvent(QEvent::MouseMove, pos, Qt::NoButton,
+                                               QApplication::mouseButtons(),
+                                               QApplication::keyboardModifiers());
+            QCoreApplication::postEvent(theMainWindow->getCanvas(), ev);
+        }
+    }
+}
+
+
 void COverlayDistance::mouseMoveEvent(QMouseEvent * e)
 {
 
@@ -251,7 +307,7 @@ void COverlayDistance::mouseMoveEvent(QMouseEvent * e)
 
     if(thePoint)
     {
-        XY pt = *thePoint;
+        projXY pt = *thePoint;
         map.convertRad2Pt(pt.u, pt.v);
         pos1 -= QPoint(pt.u - 24, pt.v - 24);
 
@@ -289,12 +345,12 @@ void COverlayDistance::mouseMoveEvent(QMouseEvent * e)
             double u1, v1, u2, v2;
             IMap& map = CMapDB::self().getMap();
 
-            if(*thePoint == points.last())
+            if(points.size() > 1 && *thePoint == points.last())
             {
                 u1 = (points.end() - 2)->u;
                 v1 = (points.end() - 2)->v;
             }
-            else if(*thePoint == points.first())
+            else if(points.size() > 1 && *thePoint == points.first())
             {
                 u1 = (points.begin() + 1)->u;
                 v1 = (points.begin() + 1)->v;
@@ -332,6 +388,29 @@ void COverlayDistance::mouseMoveEvent(QMouseEvent * e)
             {
                 GPS_Math_SubPolyline(pt1, pt2, 10, leadline, subline);
             }
+
+            QRect r = theMainWindow->getCanvas()->rect();
+
+            int w = r.width() / 10;
+            int h = r.height() / 10;
+
+            if(e->pos().x() < (r.left() + w))
+            {
+                theMainWindow->getCanvas()->move(CCanvas::eMoveLeftSmall);
+            }
+            else if(e->pos().x() > (r.right() - w))
+            {
+                theMainWindow->getCanvas()->move(CCanvas::eMoveRightSmall);
+            }
+
+            if(e->pos().y() < (r.top() + h))
+            {
+                theMainWindow->getCanvas()->move(CCanvas::eMoveUpSmall);
+            }
+            else if(e->pos().y() > (r.bottom() - h))
+            {
+                theMainWindow->getCanvas()->move(CCanvas::eMoveDownSmall);
+            }
         }
     }
 
@@ -380,7 +459,9 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
                     points.push_back(pt);
 
                 }
-                thePoint = &points.last();
+                thePointBefor   = &(*(points.end() - 2));
+                thePoint        = &points.last();
+                thePointAfter   = 0;
             }
             else if(*thePoint == points.first() && addType == eAtEnd)
             {
@@ -415,7 +496,9 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
 
                     points.push_front(pt);
                 }
-                thePoint = &points.first();
+                thePointBefor   = 0;
+                thePoint        = &points.first();
+                thePointAfter   = &(*(points.begin() + 1));
             }
             else if(addType != eNone)
             {
@@ -485,7 +568,9 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
                 map.convertPt2Rad(pt.u, pt.v);
                 points.insert(idx,pt);
 
-                thePoint    = &points[idx];
+                thePointBefor   = idx ? &points[idx - 1] : 0;
+                thePoint        = &points[idx];
+                thePointAfter   = (idx + 1) == points.count() ? 0 : &points[idx + 1];
             }
             else
             {
@@ -496,9 +581,11 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
 
                 *thePoint = pt;
 
-                doMove      = false;
-                addType     = eNone;
-                thePoint    = 0;
+                doMove          = false;
+                addType         = eNone;
+                thePoint        = 0;
+                thePointBefor   = 0;
+                thePointAfter   = 0;
 
             }
 
@@ -507,10 +594,10 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
             calcDistance();
             theMainWindow->getCanvas()->update();
 
-            if(addType == eNone)
-            {
+            //if(addType == eNone)   // why?
+            //{
                 emit sigChanged();
-            }
+            //}
             return;
         }
 
@@ -528,7 +615,7 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
         QPoint pos1 = e->pos();
 
 
-        XY pt = *thePoint;
+        projXY pt = *thePoint;
         map.convertRad2Pt(pt.u, pt.v);
         pos1 -= QPoint(pt.u - 24, pt.v - 24);
 
@@ -546,8 +633,10 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
                 COverlayDB::self().delOverlays(keys);
             }
             calcDistance();
-            doFuncWheel = false;
-            thePoint    = 0;
+            doFuncWheel     = false;
+            thePoint        = 0;
+            thePointBefor   = 0;
+            thePointAfter   = 0;
 
             emit sigChanged();
         }
@@ -571,10 +660,13 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
             map.convertPt2Rad(pt.u, pt.v);
             points.insert(idx,pt);
 
-            thePoint    = &points[idx];
-            doMove      = true;
-            addType     = eBefore;
-            doFuncWheel = false;
+            thePointBefor   = idx ? &points[idx - 1] : 0;
+            thePoint        = &points[idx];
+            thePointAfter   = (idx + 1) == points.count() ? 0 : &points[idx + 1];
+
+            doMove          = true;
+            addType         = eBefore;
+            doFuncWheel     = false;
 
             theMainWindow->getCanvas()->update();
         }
@@ -592,48 +684,30 @@ void COverlayDistance::mousePressEvent(QMouseEvent * e)
             map.convertPt2Rad(pt.u, pt.v);
             points.insert(idx,pt);
 
-            thePoint    = &points[idx];
-            doMove      = true;
-            addType     = eAfter;
-            doFuncWheel = false;
+            thePointBefor   = idx ? &points[idx - 1] : 0;
+            thePoint        = &points[idx];
+            thePointAfter   = (idx + 1) == points.count() ? 0 : &points[idx + 1];
+
+            doMove          = true;
+            addType         = eAfter;
+            doFuncWheel     = false;
 
             theMainWindow->getCanvas()->update();
 
         }
         else
         {
-            doFuncWheel = false;
-            thePoint    = 0;
+            doFuncWheel     = false;
+            thePoint        = 0;
+            thePointBefor   = 0;
+            thePointAfter   = 0;
+
         }
     }
     else if(e->button() == Qt::RightButton)
     {
-        if(doMove)
-        {
-
-            if(addType != eNone)
-            {
-                points.removeOne(*thePoint);
-            }
-            else
-            {
-                *thePoint = savePoint;
-            }
-            thePoint = 0;
-
-            calcDistance();
-            theMainWindow->getCanvas()->update();
-            emit sigChanged();
-
-            doMove      = false;
-            addType     = eNone;
-            subline.clear();
-
-            COverlayDB::self().emitSigChanged();
-
-            QApplication::restoreOverrideCursor();
-            return;
-        }
+        looseFocus();
+        return;
     }
 
     selectedPoints.clear();
@@ -697,7 +771,7 @@ void COverlayDistance::drawArrows(const QPolygon& line, const QRect& viewport, Q
             {
                 if(0 != pt.x() - pt1.x() && (pt.y() - pt1.y()))
                 {
-                    heading = ( atan2((double)(pt.y() - pt1.y()), (double)(pt.x() - pt1.x())) * 180.) / PI;
+                    heading = ( atan2((double)(pt.y() - pt1.y()), (double)(pt.x() - pt1.x())) * 180.) / M_PI;
 
                     p.save();
                     // draw arrow between bullets
@@ -726,7 +800,7 @@ void COverlayDistance::draw(QPainter& p, const QRect& viewport)
     QPixmap icon(":/icons/small_bullet_darkgray.png");
     QPixmap icon_red(":/icons/small_bullet_red.png");
     QPixmap icon_BigRed(":/icons/bullet_red.png");
-    XY pt1, pt2;
+    projXY pt1, pt2;
     QPoint pt;
 
 
@@ -936,14 +1010,61 @@ void COverlayDistance::draw(QPainter& p, const QRect& viewport)
     // overlay points with the selected point icon
     foreach(i, selectedPoints)
     {
-        XY pt = points[i];
+        projXY pt = points[i];
 
         map.convertRad2Pt(pt.u, pt.v);
         p.drawPixmap(pt.u - 6, pt.v - 6, icon_BigRed);
     }
 
+    // draw distance information to neighbour points
+    if(thePointBefor && subline.isEmpty())
+    {
+        drawDistanceInfo(*thePointBefor, *thePoint, p, map);
+    }
+
+    if(thePointAfter && subline.isEmpty())
+    {
+        drawDistanceInfo(*thePoint, *thePointAfter, p, map);
+    }
+
 }
 
+void COverlayDistance::drawDistanceInfo(projXY p1, projXY p2, QPainter& p, IMap& map)
+{
+    QString val, unit, str;
+    double a1, a2, dist;
+
+    dist = ::distance(p1, p2, a1, a2);
+    IUnit::self().meter2distance(dist, val, unit);
+    str = QString("%1 %2 %3\260").arg(val).arg(unit).arg(a2,0,'f',0);
+
+    map.convertRad2Pt(p1.u, p1.v);
+    map.convertRad2Pt(p2.u, p2.v);
+
+    QFontMetrics fm(CResources::self().getMapFont());
+    qint32 pixel = sqrt((p2.u - p1.u)*(p2.u - p1.u) + (p2.v - p1.v)*(p2.v - p1.v));
+
+    if(fm.width(str) > pixel)
+    {
+        return;
+    }
+
+    p.save();
+    p.translate(p1.u + (p2.u - p1.u) / 2, p1.v + (p2.v - p1.v) / 2);
+    if(a2 > 0)
+    {
+        p.rotate(a2 - 90);
+    }
+    else
+    {
+        p.rotate(a2 + 90);
+    }
+
+    CCanvas::drawText(str, p, QPoint(0,0));
+
+    p.restore();
+
+}
 
 
 
@@ -1006,7 +1127,7 @@ void COverlayDistance::slotToTrack()
     if(points.isEmpty()) return;
 
     double dist, d, delta = 10.0, a1 , a2;
-    XY pt1, pt2, ptx;
+    projXY pt1, pt2, ptx;
     CTrack::pt_t pt;
     CDlgConvertToTrack::EleMode_e eleMode;
 
@@ -1111,7 +1232,7 @@ void COverlayDistance::slotToRoute()
 
     route->setName(name);
 
-    XY pt;
+    projXY pt;
 
     for(int i = 0; i < points.count(); ++i)
     {
@@ -1156,7 +1277,7 @@ void COverlayDistance::makeVisible()
     double west  = +180.0 * DEG_TO_RAD;
     double east  = -180.0 * DEG_TO_RAD;
 
-    XY pt;
+    projXY pt;
     foreach(pt, points)
     {
         if(pt.u < west)  west  = pt.u;
@@ -1170,17 +1291,33 @@ void COverlayDistance::makeVisible()
     emit sigChanged();
 }
 
-
 void COverlayDistance::looseFocus()
 {
+    if(doMove)
+    {
+        if(addType != eNone)
+        {
+            points.removeOne(*thePoint);
+        }
+        else
+        {
+            *thePoint = savePoint;
+        }
+    }
+
     if(doSpecialCursor)
     {
         QApplication::restoreOverrideCursor();
         doSpecialCursor = false;
     }
+
     doMove          = false;
     addType         = eNone;
     doFuncWheel     = false;
+
+    subline.clear();
+    calcDistance();
+    emit sigChanged();
 }
 
 
@@ -1191,7 +1328,7 @@ QRectF COverlayDistance::getBoundingRectF()
     double west  = +180.0 * DEG_TO_RAD;
     double east  = -180.0 * DEG_TO_RAD;
 
-    XY pt;
+    projXY pt;
     foreach(pt, points)
     {
         if(pt.u < west)  west  = pt.u;
@@ -1224,10 +1361,12 @@ void COverlayDistance::delPointsByIdx(const QList<int>& idx)
         }
     }
 
-    thePoint    = 0;
-    doMove      = false;
-    addType     = eNone;
-    doFuncWheel = false;
+    thePoint        = 0;
+    thePointBefor   = 0;
+    thePointAfter   = 0;
+    doMove          = false;
+    addType         = eNone;
+    doFuncWheel     = false;
 
     selectedPoints.clear();
 
