@@ -19,6 +19,8 @@
 
 #include "CResources.h"
 #include "CDeviceGarmin.h"
+#include "CDeviceGarminBulk.h"
+#include "CDeviceTwoNav.h"
 #include "CDeviceQLandkarteM.h"
 #ifdef HS_MIKROKOPTER
 #include "CDeviceMikrokopter.h"
@@ -33,6 +35,14 @@
 #include "CUnitImperial.h"
 #include "CMapTDB.h"
 #include "config.h"
+#include "CSettings.h"
+
+#ifndef Q_OS_WIN32
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <pwd.h>
+#include <unistd.h>
+#endif
 
 #include <QtGui>
 
@@ -47,7 +57,6 @@ CResources::CResources(QObject * parent)
 , m_device(0)
 , m_devIPPort(4242)
 , m_flipMouseWheel(false)
-#ifdef HAS_GEODB
 , m_useGeoDB(true)
 , m_saveGeoDBOnExit(false)
 , m_saveGeoDBMinutes(5)
@@ -56,7 +65,6 @@ CResources::CResources(QObject * parent)
 #else
 , m_pathGeoDB(QDir::home().filePath(CONFIGDIR))
 #endif
-#endif
 , m_showTrackProfile(true)
 , m_showNorth(true)
 , m_showScale(true)
@@ -65,11 +73,13 @@ CResources::CResources(QObject * parent)
 , m_useAntiAliasing(true)
 , m_reducePoiIcons(true)
 , m_WptTextColor(Qt::black)
-
+, m_pathMapCache(QDir::temp().filePath("qlandkartegt/cache"))
+, m_sizeMapCache(100)
+, m_expireMapCache(8)
 {
     m_self = this;
 
-    QSettings cfg;
+    SETTINGS;
 
     QString family      = cfg.value("environment/mapfont/family","Arial").toString();
     int size            = cfg.value("environment/mapfont/size",8).toInt();
@@ -81,18 +91,19 @@ CResources::CResources(QObject * parent)
 
     //m_doMetric        = cfg.value("environment/doMetric",true).toBool();
     m_flipMouseWheel    = cfg.value("environment/flipMouseWheel",m_flipMouseWheel).toBool();
-#ifdef HAS_GEODB
     m_useGeoDB          = cfg.value("environment/GeoDB",m_useGeoDB).toBool();
     m_saveGeoDBOnExit   = cfg.value("environment/saveGeoDBOnExit",m_saveGeoDBOnExit).toBool();
     m_saveGeoDBMinutes  = cfg.value("environment/saveGeoDBMinutes",m_saveGeoDBMinutes).toUInt();
     m_pathGeoDB         = QDir(cfg.value("environment/pathGeoDB", m_pathGeoDB.absolutePath()).toString());
-#endif
 
     m_useHttpProxy      = cfg.value("network/useProxy",m_useHttpProxy).toBool();
     m_httpProxy         = cfg.value("network/proxy/url",m_httpProxy).toString();
     m_httpProxyPort     = cfg.value("network/proxy/port",m_httpProxyPort).toUInt();
 
-    emit sigProxyChanged();
+    if(m_useHttpProxy)
+      QNetworkProxy::setApplicationProxy(QNetworkProxy(QNetworkProxy::HttpProxy,m_httpProxy,m_httpProxyPort));
+    else
+      QNetworkProxy::setApplicationProxy(QNetworkProxy(QNetworkProxy::NoProxy));
 
     m_devKey            = cfg.value("device/key",m_devKey).toString();
     m_devIPAddress      = cfg.value("device/ipAddr",m_devIPAddress).toString();
@@ -131,13 +142,13 @@ CResources::CResources(QObject * parent)
         unit = new CUnitMetric(this);
     }
 
-    m_showTrackProfile  = cfg.value("environment/showTrackProfile",m_showTrackProfile).toBool();
-    m_showNorth     = cfg.value("environment/showNorth",m_showNorth).toBool();
-    m_showScale     = cfg.value("environment/showScale",m_showScale).toBool();
-    m_showToolTip   = cfg.value("environment/showToolTip",m_showToolTip).toBool();
-    m_showZoomLevel = cfg.value("environment/showZoomLevel",m_showZoomLevel).toBool();
-    m_useAntiAliasing = cfg.value("environment/useAntiAliasing",m_useAntiAliasing).toBool();
-    m_reducePoiIcons = cfg.value("environment/reducePoiIcons",m_reducePoiIcons).toBool();
+    m_showTrackProfile = cfg.value("environment/showTrackProfile",m_showTrackProfile).toBool();
+    m_showNorth        = cfg.value("environment/showNorth",m_showNorth).toBool();
+    m_showScale        = cfg.value("environment/showScale",m_showScale).toBool();
+    m_showToolTip      = cfg.value("environment/showToolTip",m_showToolTip).toBool();
+    m_showZoomLevel    = cfg.value("environment/showZoomLevel",m_showZoomLevel).toBool();
+    m_useAntiAliasing  = cfg.value("environment/useAntiAliasing",m_useAntiAliasing).toBool();
+    m_reducePoiIcons   = cfg.value("environment/reducePoiIcons",m_reducePoiIcons).toBool();
 
     m_WptTextColor = QColor(cfg.value("environment/wptTextColor", m_WptTextColor.name()).toString());
 
@@ -155,12 +166,41 @@ CResources::CResources(QObject * parent)
     QPixmap(":/webstuff/frame_bottom_right.png").save(dirWeb.filePath("frame_bottom_right.png"));
     QPixmap(":/webstuff/scale.png").save(dirWeb.filePath("scale.png"));
 
+
+    QString cacheFolder;
+#ifndef Q_OS_WIN32
+    const char *envCache = getenv("QLGT_CACHE");
+
+    if (envCache)
+    {
+        cacheFolder = envCache;
+    }
+    else
+    {
+#ifndef Q_WS_MAC
+        struct passwd * userInfo = getpwuid(getuid());
+        cacheFolder = QDir::tempPath() + "/qlandkarteqt-" + userInfo->pw_name + "/cache/";
+#else
+        // Mac OS X: points to /Users/<user name>/Library/Caches/QLandkarteGT/QLandkarteGT
+        cacheFolder = QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
+#endif
+    }
+#else
+    cacheFolder = m_pathMapCache.absolutePath();
+#endif
+
+    m_pathMapCache = QDir(cfg.value("network/mapcache/path", cacheFolder).toString());
+    m_sizeMapCache = cfg.value("network/mapcache/size", m_sizeMapCache).toInt();
+    m_expireMapCache = cfg.value("network/mapcache/expire", m_expireMapCache).toInt();
+
 }
 
 
 CResources::~CResources()
 {
-    QSettings cfg;
+    SETTINGS;
+
+    cfg.setValue("path/maps",pathMaps);
 
     cfg.setValue("environment/mapfont/family",m_mapfont.family());
     cfg.setValue("environment/mapfont/size",m_mapfont.pointSize());
@@ -168,12 +208,10 @@ CResources::~CResources()
     cfg.setValue("environment/mapfont/italic",m_mapfont.italic());
 
     cfg.setValue("environment/flipMouseWheel",m_flipMouseWheel);
-#ifdef HAS_GEODB
     cfg.setValue("environment/GeoDB",m_useGeoDB);
     cfg.setValue("environment/saveGeoDBOnExit",m_saveGeoDBOnExit);
     cfg.setValue("environment/saveGeoDBMinutes",m_saveGeoDBMinutes);
     cfg.setValue("environment/pathGeoDB",m_pathGeoDB.absolutePath());
-#endif
     cfg.setValue("network/useProxy",m_useHttpProxy);
     cfg.setValue("network/proxy/url",m_httpProxy);
     cfg.setValue("network/proxy/port",m_httpProxyPort);
@@ -205,6 +243,10 @@ CResources::~CResources()
     cfg.setValue("environment/reducePoiIcons",m_reducePoiIcons);
 
     cfg.setValue("environment/wptTextColor", m_WptTextColor.name());
+
+    cfg.setValue("network/mapcache/path", m_pathMapCache.absolutePath());
+    cfg.setValue("network/mapcache/size", m_sizeMapCache);
+    cfg.setValue("network/mapcache/expire", m_expireMapCache);
 }
 
 
@@ -271,6 +313,14 @@ IDevice * CResources::device()
             m_device = new CDeviceGPSD(this);
         }
 #endif
+        else if(m_devKey == "Garmin Mass Storage")
+        {
+            m_device = new CDeviceGarminBulk(this);
+        }
+        else if(m_devKey == "TwoNav")
+        {
+            m_device = new CDeviceTwoNav(this);
+        }
 
         connect(m_device, SIGNAL(sigLiveLog(const CLiveLog&)), &CLiveLogDB::self(), SLOT(slotLiveLog(const CLiveLog&)));
     }
