@@ -57,12 +57,10 @@ class CResources : public QObject
         /// root path of all maps
         QString pathMaps;
 
-#ifdef HAS_GEODB
         bool useGeoDB(){return m_useGeoDB;}
         bool saveGeoDBOnExit(){return m_saveGeoDBOnExit;}
         quint32 saveGeoDBMinutes(){return m_saveGeoDBMinutes;}
         QDir pathGeoDB(){return m_pathGeoDB;}
-#endif
         bool flipMouseWheel(){return m_flipMouseWheel;}
         bool showTrackProfilePreview(){return m_showTrackProfile;}
         bool showNorthIndicator(){return m_showNorth;}
@@ -75,8 +73,11 @@ class CResources : public QObject
 
         QColor wptTextColor(){return m_WptTextColor;}
 
+        QDir getPathMapCache(){return m_pathMapCache;}
+        int getSizeMapCache(){return m_sizeMapCache;}
+        int getExpireMapCache(){return m_expireMapCache;}
+
         signals:
-        void sigProxyChanged();
         void sigDeviceChanged();
 
     private:
@@ -130,13 +131,10 @@ class CResources : public QObject
         /// play sound after finishing transfers
         bool m_playSound;
 
-#ifdef HAS_GEODB
         bool m_useGeoDB;
         bool m_saveGeoDBOnExit;
         quint32 m_saveGeoDBMinutes;
         QDir m_pathGeoDB;
-#endif
-
 
         bool m_showTrackProfile;
         bool m_showNorth;
@@ -150,5 +148,9 @@ class CResources : public QObject
 
         /// unit translator object
         QPointer<IUnit> unit;
+
+        QDir m_pathMapCache;
+        int  m_sizeMapCache;
+        int  m_expireMapCache;
 };
 #endif                           //CRESOURCES_H
