@@ -24,9 +24,13 @@
 #include "IUnit.h"
 #include "CMainWindow.h"
 #include "CCanvas.h"
+#include "CWptDB.h"
+#include "CTrackDB.h"
+#include "CSettings.h"
 
 #include <QtGui>
 #include <QtNetwork/QHttp>
+#include <QtNetwork/QNetworkProxy>
 
 #ifndef _MKSTR_1
 #define _MKSTR_1(x)    #x
@@ -91,6 +95,14 @@ QDataStream& operator >>(QDataStream& s, CTrack& track)
                 s1 >> track.comment;
                 s1 >> track.colorIdx;
                 s1 >> track.parentWpt;
+                if(!s1.atEnd())
+                {
+                    s1 >> track.doScaleWpt2Track;
+                }
+                if(!s1.atEnd())
+                {
+                    s1 >> track.cntMedianFilterApplied;
+                }
 
                 track.setColor(track.colorIdx);
                 track.setKey(key);
@@ -265,6 +277,8 @@ QDataStream& operator <<(QDataStream& s, CTrack& track)
     s1 << track.comment;
     s1 << track.colorIdx;
     s1 << track.getParentWpt();
+    s1 << track.doScaleWpt2Track;
+    s1 << track.cntMedianFilterApplied;
 
     entries << entryBase;
 
@@ -461,6 +475,7 @@ void operator <<(QFile& f, CTrack& track)
     f.close();
 }
 
+#define DEFAULT_COLOR 4
 
 const QColor CTrack::lineColors[] =
 {
@@ -531,21 +546,25 @@ CTrack::CTrack(QObject * parent)
 : IItem(parent)
 , color(Qt::darkBlue)
 , bullet(":/icons/small_bullet_darkblue.png")
-, colorIdx(4)
+, colorIdx(DEFAULT_COLOR)
 , highlight(false)
 , traineeData(false)
 , ext1Data(false)
 , firstTime(true)
 , m_hide(false)
+, doScaleWpt2Track(Qt::PartiallyChecked	)
 , geonames(0)
 , visiblePointCount(0)
+, cntMedianFilterApplied(0)
 {
     ref = 1;
 
-    setColor(4);
-
-    slotSetupLink();
-    connect(&CResources::self(), SIGNAL(sigProxyChanged()), this, SLOT(slotSetupLink()));
+    setColor(DEFAULT_COLOR);
+    geonames = new QHttp(this);
+    geonames->setProxy(QNetworkProxy(QNetworkProxy::DefaultProxy));
+    geonames->setHost("ws.geonames.org");
+    connect(geonames,SIGNAL(requestStarted(int)),this,SLOT(slotRequestStarted(int)));
+    connect(geonames,SIGNAL(requestFinished(int,bool)),this,SLOT(slotRequestFinished(int,bool)));
 }
 
 
@@ -554,25 +573,6 @@ CTrack::~CTrack()
 
 }
 
-void CTrack::slotSetupLink()
-{
-    QString url;
-    quint16 port;
-    bool enableProxy;
-
-    enableProxy = CResources::self().getHttpProxy(url,port);
-
-    if(geonames) delete geonames;
-    geonames = new QHttp(this);
-    if(enableProxy)
-    {
-        geonames->setProxy(url,port);
-    }
-    geonames->setHost("ws.geonames.org");
-    connect(geonames,SIGNAL(requestStarted(int)),this,SLOT(slotRequestStarted(int)));
-    connect(geonames,SIGNAL(requestFinished(int,bool)),this,SLOT(slotRequestFinished(int,bool)));
-}
-
 void CTrack::replaceElevationByLocal()
 {
     qDebug() << "CTrack::replaceElevationByLocal()";
@@ -589,6 +589,9 @@ void CTrack::replaceElevationByLocal()
 
 void CTrack::replaceElevationByRemote()
 {
+    SETTINGS;
+    QString username = cfg.value("geonames/username","demo").toString();
+
     int idx = 0, id;
     const int size = track.size();
 
@@ -609,6 +612,7 @@ void CTrack::replaceElevationByRemote()
         url.setPath("/srtm3");
         url.addQueryItem("lats",lats.join(","));
         url.addQueryItem("lngs",lngs.join(","));
+        url.addQueryItem("username",username);
         id = geonames->get(url.toEncoded( ));
 
         id2idx[id] = idx;
@@ -634,7 +638,7 @@ void CTrack::slotRequestFinished(int id, bool error)
 
     QString asw = geonames->readAll().simplified();
 
-//    qDebug() << asw;
+    qDebug() << asw;
 
     if(asw.isEmpty())
     {
@@ -651,8 +655,9 @@ void CTrack::slotRequestFinished(int id, bool error)
         {
             if(idx < track.size())
             {
-                track[idx++].ele    = val.toDouble();
-                track[idx++]._ele   = val.toDouble();
+                track[idx].ele    = val.toDouble();
+                track[idx]._ele   = val.toDouble();
+                idx++;
             }
         }
 
@@ -693,10 +698,36 @@ QRectF CTrack::getBoundingRectF()
     return QRectF(QPointF(west,north),QPointF(east,south));
 }
 
+void CTrack::setColor(const QColor& c)
+{
+    int n;
+    int N = sizeof(lineColors)/sizeof(QColor);
+
+    for(n = 0; n < N; n++)
+    {
+        if(lineColors[n] == c)
+        {
+            colorIdx    = n;
+            color       = lineColors[n];
+            bullet      = QPixmap(bulletColors[n]);
+            break;
+        }
+    }
+
+    if(n == N)
+    {
+        colorIdx    = DEFAULT_COLOR;
+        color       = lineColors[DEFAULT_COLOR];
+        bullet      = QPixmap(bulletColors[DEFAULT_COLOR]);
+    }
+
+    setIcon(color.name());
+
+}
 
 void CTrack::setColor(unsigned i)
 {
-    if(i>16) i = 4;
+    if(i>16) i = DEFAULT_COLOR;
     colorIdx    = i;
     color       = lineColors[i];
     bullet      = QPixmap(bulletColors[i]);
@@ -826,8 +857,8 @@ void CTrack::rebuild(bool reindex)
             pt2->delta      = 0;
             pt2->speed      = -1;
             pt2->distance   = 0;
-            pt1->ascend     = 0;
-            pt1->descend    = 0;
+            pt2->ascend     = 0;
+            pt2->descend    = 0;
             continue;
         }
 
@@ -839,7 +870,7 @@ void CTrack::rebuild(bool reindex)
             dt = pt2->timestamp - pt1->timestamp;
         }
 
-        XY p1,p2;
+        projXY p1,p2;
         p1.u = DEG_TO_RAD * pt1->lon;
         p1.v = DEG_TO_RAD * pt1->lat;
         p2.u = DEG_TO_RAD * pt2->lon;
@@ -996,6 +1027,39 @@ QDateTime CTrack::getEndTimestamp()
     return QDateTime();
 }
 
+float CTrack::getStartElevation()
+{
+    QList<CTrack::pt_t>& trkpts           = track;
+    QList<CTrack::pt_t>::iterator trkpt   = trkpts.begin();
+    while(trkpt != trkpts.end())
+    {
+        if(trkpt->flags & pt_t::eDeleted)
+        {
+            ++trkpt;
+            continue;
+        }
+        return trkpt->ele;
+    }
+    return WPT_NOFLOAT;
+}
+
+float CTrack::getEndElevation()
+{
+    QList<CTrack::pt_t>& trkpts           = track;
+    QList<CTrack::pt_t>::iterator trkpt   = trkpts.end() - 1;
+    while(trkpt > trkpts.begin())
+    {
+        if(trkpt->flags & pt_t::eDeleted)
+        {
+            --trkpt;
+            continue;
+        }
+        return trkpt->ele;
+    }
+    return WPT_NOFLOAT;
+}
+
+
 /// get a summary of item's data to display on screen or in the toolview
 QString CTrack::getInfo()
 {
@@ -1051,7 +1115,7 @@ QString CTrack::getInfo()
     IUnit::self().meter2elevation(getAscend(), val1, unit1);
     IUnit::self().meter2elevation(getDescend(), val2, unit2);
 
-    str += tr("\n%1%2 %3, %4%5 %6").arg(QChar(0x2191)).arg(val1).arg(unit1).arg(QChar(0x2193)).arg(val2).arg(unit2);
+    str += tr("\n%1%2 %3, %4%5 %6").arg(QChar(0x2197)).arg(val1).arg(unit1).arg(QChar(0x2198)).arg(val2).arg(unit2);
 
     return str;
 }
@@ -1138,6 +1202,171 @@ QString CTrack::getTrkPtInfo(pt_t& trkpt)
     return str;
 }
 
+void CTrack::setDoScaleWpt2Track(Qt::CheckState state)
+{
+    doScaleWpt2Track = state;
+    CTrackDB::self().emitSigModified();
+    CTrackDB::self().emitSigChanged();
+}
+
+void CTrack::scaleWpt2Track(QList<wpt_t>& wpts)
+{
+    CWptDB& wptdb = CWptDB::self();
+    if(wptdb.count() == 0 ) return;
+    IMap& map = CMapDB::self().getMap();
+
+    wpts.clear();
+
+    if(doScaleWpt2Track == Qt::Unchecked)
+    {
+        return;
+    }
+
+    if(doScaleWpt2Track == Qt::PartiallyChecked	)
+    {
+        if(wptdb.count() > 1000 || track.count() > 10000)
+        {
+            QString msg = tr(
+                    "You are trying to find waypoints along a track with %1 waypoints and a track of size %2. "
+                    "This can be a very time consuming operation. Go on?\n\n"
+                    "Your selection will be stored in the track's data. You can save it along with the data. "
+                    "To change the selection use the checkbox in the track edit dialog."
+                    ).arg(wptdb.count()).arg(track.count());
+
+            QMessageBox::Button res = QMessageBox::warning(0,tr("Warning..."), msg, QMessageBox::Ok| QMessageBox::Abort, QMessageBox::Ok);
+
+            if(res == QMessageBox::Abort)
+            {
+                doScaleWpt2Track = Qt::Unchecked;
+                CTrackDB::self().emitSigModified();
+                CTrackDB::self().emitSigChanged();
+                return;
+            }
+            else
+            {
+                doScaleWpt2Track = Qt::Checked;
+                CTrackDB::self().emitSigModified();
+                CTrackDB::self().emitSigChanged();
+            }
+        }
+    }
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+    QMap<QString,CWpt*>::const_iterator w = wptdb.begin();
+    while(w != wptdb.end())
+    {
+        wpt_t wpt;
+        wpt.wpt    = (*w);
+        wpt.x      = wpt.wpt->lon * DEG_TO_RAD;
+        wpt.y      = wpt.wpt->lat * DEG_TO_RAD;
+
+        map.convertRad2M(wpt.x, wpt.y);
+
+        wpts << wpt;
+
+        ++w;
+    }
+
+    QList<pt_t>& trkpts                 = getTrackPoints();
+    QList<pt_t>::const_iterator trkpt   = trkpts.begin();
+    while(trkpt != trkpts.end())
+    {
+        if(trkpt->flags & pt_t::eDeleted)
+        {
+            ++trkpt;
+            continue;
+        }
+
+        double x = trkpt->lon * DEG_TO_RAD;
+        double y = trkpt->lat * DEG_TO_RAD;
+        map.convertRad2M(x, y);
+
+        QList<wpt_t>::iterator wpt = wpts.begin();
+        while(wpt != wpts.end())
+        {
+            double d = (x - wpt->x) * (x - wpt->x) + (y - wpt->y) * (y - wpt->y);
+            if(d < wpt->d)
+            {
+                wpt->d      = d;
+                wpt->trkpt  = *trkpt;
+            }
+            ++wpt;
+        }
+        ++trkpt;
+    }
+
+    QApplication::restoreOverrideCursor();
+}
+
+void CTrack::medianFilter(QProgressDialog& progress)
+{
+    qint32 len = 5 + (cntMedianFilterApplied<<1);
+
+    QList<float> window;
+    for(int i = 0; i<len; i++)
+    {
+        window << 0.0;
+    }
+
+    QList<CTrack::pt_t>& trkpts = getTrackPoints();
+    QList<float> ele;
+
+    if(cntMedianFilterApplied)
+    {
+        foreach(const CTrack::pt_t& pt, trkpts)
+        {
+            ele << pt.ele;
+        }
+    }
+    else
+    {
+        foreach(const CTrack::pt_t& pt, trkpts)
+        {
+            ele << pt._ele;
+        }
+    }
+
+    for(int i = (len>>1); i < (ele.size()-(len>>1)); i++)
+    {
+        // apply median filter over all trackpoints
+        for(int n = 0; n < len; n++)
+        {
+            window[n] = ele[i - (len>>1) + n];
+        }
+
+        qSort(window);
+        trkpts[i].ele = window[(len>>1)];
+
+        progress.setValue(i);
+        qApp->processEvents();
+        if (progress.wasCanceled())
+        {
+            return;
+        }
+    }
+
+    cntMedianFilterApplied++;
+}
+
+void CTrack::reset()
+{
+    QList<CTrack::pt_t>& trkpts           = getTrackPoints();
+    QList<CTrack::pt_t>::iterator trkpt   = trkpts.begin();
+    while(trkpt != trkpts.end())
+    {
+        trkpt->flags &= ~CTrack::pt_t::eDeleted;
+        trkpt->lon = trkpt->_lon;
+        trkpt->lat = trkpt->_lat;
+        trkpt->ele = trkpt->_ele;
+
+        ++trkpt;
+    }
+
+    cntMedianFilterApplied = 0;
+
+    rebuild(true);
+}
+
 QDataStream& operator >>(QDataStream& s, CFlags& flag)
 {
     quint32 f;
