@@ -27,6 +27,7 @@
 #include <QDateTime>
 #include <QMap>
 #include <QList>
+#include <QPointer>
 #include "CWpt.h"
 #include "IItem.h"
 
@@ -35,8 +36,10 @@
 #endif
 
 #define MAX_TRACK_SIZE 10000
+#define WPT_TO_TRACK_DIST 2500
 
 class QHttp;
+class QProgressDialog;
 
 class CFlags
 {
@@ -92,7 +95,7 @@ class CTrack : public IItem
                 _lon(WPT_NOFLOAT),_lat(WPT_NOFLOAT),_ele(WPT_NOFLOAT),
                 flags(0), px_valid(FALSE), dem(WPT_NOFLOAT), editItem(NULL){}
 
-            bool operator==(const pt_t& pt){return pt.idx == idx;}
+            bool operator==(const pt_t& pt) const {return pt.idx == idx;}
 
             /// index counter for easy QVector access
             qint32  idx;
@@ -159,7 +162,6 @@ class CTrack : public IItem
             /// elevation [m]
             float   _ele;
 
-
             /// display flags
             CFlags flags;
             /// the current location in pixel
@@ -169,35 +171,47 @@ class CTrack : public IItem
             float  dem;
 
             /// QTreeWidgetItem
-            QObject * editItem;
+            QPointer<QObject> editItem;
         };
 
+        struct wpt_t
+        {
+            wpt_t() : wpt(0), d(1e25f), x(0), y(0) {}
+            CWpt * wpt;
+            double d;
+            double x;
+            double y;
+            pt_t trkpt;
+        };
+
+
         /// set color by id
         void setColor(unsigned i);
+        void setColor(const QColor& c);
         /// get QT color
-        const QColor& getColor(){return color;}
-        const QPixmap& getBullet(){return bullet;}
-        unsigned getColorIdx(){return colorIdx;}
+        const QColor& getColor() const {return color;}
+        const QPixmap& getBullet() const {return bullet;}
+        unsigned getColorIdx() const {return colorIdx;}
 
         /// set the highlight flag
-        void setHighlight(bool yes){highlight = yes;}
+        void setHighlight(bool yes) {highlight = yes;}
         /// get the value of the highlight flag
-        bool isHighlighted(){return highlight;}
+        bool isHighlighted() const {return highlight;}
 
         /// append point to track
         CTrack& operator<<(pt_t& pt);
         /// rebuild secondary track data from primary
         void rebuild(bool reindex);
         /// get list of track points
-        QList<pt_t>& getTrackPoints(){return track;}
+        QList<pt_t>& getTrackPoints() {return track;}
         /// get polyline representation of track
-        QPolygon& getPolyline(){return polyline;}
+        QPolygon& getPolyline() {return polyline;}
         /// get the total distance of the track in [m]
-        double getTotalDistance(){return totalDistance;}
+        double getTotalDistance() const {return totalDistance;}
         /// get the total time covered by the track in seconds
-        quint32 getTotalTime(){return totalTime;}
+        quint32 getTotalTime() const {return totalTime;}
         /// get the total time while moving around
-        quint32 getTotalTimeMoving(){return totalTimeMoving;}
+        quint32 getTotalTimeMoving() const {return totalTimeMoving;}
         /// select tarckpoint by index
         void setPointOfFocus(int idx, bool eraseSelection, bool moveMap);
         /// set point of focus to a point with a given distance from start
@@ -207,14 +221,16 @@ class CTrack : public IItem
         ///
         QDateTime getEndTimestamp();
         /// get the ascend in [m]
-        double getAscend(){return totalAscend;}
+        double getAscend() const {return totalAscend;}
         /// get the descend in [m]
-        double getDescend(){return totalDescend;}
-
+        double getDescend() const {return totalDescend;}
+        /// get information string for a particular trackpoint
         QString getTrkPtInfo(pt_t& trkpt);
-
+        /// get the bounding rectangular that fits the track
         QRectF getBoundingRectF();
+        /// sort trackpoints by timestamp
         void sortByTimestamp();
+        /// combine tracks
         CTrack& operator+=(const CTrack& trk);
 
         static const QColor lineColors[];
@@ -226,7 +242,7 @@ class CTrack : public IItem
         void setExt1Data() { ext1Data = true;}
 
         void hide(bool ok);
-        bool isHidden(){return m_hide;}
+        bool isHidden() const {return m_hide;}
 
         void replaceElevationByRemote();
         void replaceElevationByLocal();
@@ -236,14 +252,26 @@ class CTrack : public IItem
         /// set the icon defined by a string
         void setIcon(const QString& str);
 
-
         void setTimestamp(quint32 ts){timestamp = ts;}
+        void scaleWpt2Track(QList<wpt_t>& wpts);
+        float getStartElevation();
+        float getEndElevation();
+
+        Qt::CheckState getDoScaleWpt2Track(){return (Qt::CheckState)doScaleWpt2Track;}
+        void setDoScaleWpt2Track(Qt::CheckState state);
+
+        /// smooth profile with a median filter
+        void medianFilter(QProgressDialog& progress);
+
+        /// reset all smoothed and purged data to it's original state
+        void reset();
+
+        quint32 getMedianFilterCount() const {return cntMedianFilterApplied;}
 
         signals:
         void sigChanged();
 
     private slots:
-        void slotSetupLink();
         void slotRequestStarted(int );
         void slotRequestFinished(int , bool error);
 
@@ -253,8 +281,6 @@ class CTrack : public IItem
         friend QDataStream& operator >>(QDataStream& s, CTrack& track);
         friend QDataStream& operator <<(QDataStream& s, CTrack& track);
 
-        void medianFilter();
-
         static QDir path;
 
         /// a track URL
@@ -296,17 +322,19 @@ class CTrack : public IItem
 
         bool traineeData;
         bool ext1Data;
-
         bool firstTime;
-
         bool m_hide;
 
+        quint32 doScaleWpt2Track;
+
         QHttp * geonames;
 
         QMap<int,int> id2idx;
 
         quint32 visiblePointCount;
 
+        quint32 cntMedianFilterApplied;
+
 };
 
 QDataStream& operator >>(QDataStream& s, CTrack& track);
