@@ -112,10 +112,10 @@ void CTrackStatProfileWidget::slotChanged()
         }
         else
         {
-            lineElev.clear();
-            lineDEM.clear();
-            marksElev.clear();
-            break;
+//            lineElev.clear();
+//            lineDEM.clear();
+//            marksElev.clear();
+//            break;
         }
 
 
@@ -132,15 +132,15 @@ void CTrackStatProfileWidget::slotChanged()
         ++trkpt;
     }
 
-    QVector<wpt_t> wpts;
-
     plot->clear();
-    addWptTags(wpts);
 
-    QVector<wpt_t>::const_iterator wpt = wpts.begin();
+    QList<CTrack::wpt_t> wpts;
+    track->scaleWpt2Track(wpts);
+
+    QList<CTrack::wpt_t>::const_iterator wpt = wpts.begin();
     while(wpt != wpts.end())
     {
-        if(wpt->d < 400)
+        if(wpt->d < WPT_TO_TRACK_DIST)
         {
             CPlotData::point_t tag;
             tag.point = QPointF(type == eOverDistance ? wpt->trkpt.distance :  (double)wpt->trkpt.timestamp, wpt->trkpt.ele);
