@@ -20,12 +20,12 @@
 #ifndef GARMIN_H
 #define GARMIN_H
 
-#include <projects.h>
+#include <proj_api.h>
 #ifdef __MINGW32__
 #undef LP
 #endif
 
 #define GARMIN_DEG(x) ((x) < 0x800000 ? (double)(x) * 360.0 / 16777216.0 : (double)((x) - 0x1000000) * 360.0 / 16777216.0)
-#define GARMIN_RAD(x) ((x) < 0x800000 ? (double)(x) * TWOPI / 16777216.0 : (double)((x) - 0x1000000) * TWOPI / 16777216.0)
+#define GARMIN_RAD(x) ((x) < 0x800000 ? (double)(x) * (2*M_PI) / 16777216.0 : (double)((x) - 0x1000000) * (2*M_PI) / 16777216.0)
 typedef quint8 quint24[3];
 #endif                           //GARMIN_H
